/*
Vsichki te znaiat:
"Otechestvoto
e v opasnost"
    Prekrasno - no
    shsto e otechestvo ?
*/
#include <iostream>
#include <cstdint>
#include <random>
#include <utility>
#include <cassert>

struct AnsT {
	uint32_t i;
	int32_t d;
	friend bool operator< (const AnsT &a, const AnsT &b) {
		return a.d < b.d;
	}
	void apply_lazy(int32_t l) {
		d += l;
	}
};

const uint32_t MAX_N = 1e5 + 10;

class Treap {
	static std::mt19937 rnd;

	static const uint32_t MAX_MEM = 6 * MAX_N;
	static Treap mem[];
	static Treap* memPtr;

	const uint32_t p;
	uint32_t lazy;
	uint32_t size;

	Treap(): p(rnd()), lazy(0), size(1), par(nullptr), l(nullptr), r(nullptr) {}

	void recalc() {
		best = cur;
		size = 1;
		best.d -= lazy;
		if (l) {
			best = std::min(best, l->best);
			size += l->size;
		}
		if (r) {
			best = std::min(best, r->best);
			size += r->size;
		}

		best.d += lazy;
	}

	Treap *par;
	Treap *l, *r;

	void cut_l() {
		if (l) {
			l->par = nullptr;
		}
	}
	void set_l(Treap *n) {
		l = n;
		if (n) {
			n->par = this;
		}
	}
	void cut_r() {
		if (r) {
			r->par = nullptr;
		}
	}
	void set_r(Treap *n) {
		r = n;
		if (n) {
			n->par = this;
		}
	}
	void push_lazy() {
		if (l) {
			l->add_lazy(lazy);
		}
		if (r) {
			r->add_lazy(lazy);
		}
		lazy = 0;
	}

	AnsT best;
	AnsT cur;
public:
	AnsT get_best() {
		return best;
	}
	AnsT get_cur() {
		return cur;
	}
	void set_cur(const AnsT &a) {
		cur = a;
		recalc();
	}
	void add_lazy(int32_t l) {
		lazy += l;
		cur.d += l;
		recalc();
	}
	void prop_lazy() {
		if (par) {
			par->prop_lazy();
		}
		push_lazy();
	}
	uint32_t get_size() const {
		return size;
	}

	
	static Treap* get_node() {
		return memPtr++;
	}

	static Treap* merge(Treap *a, Treap *b) {
		if (a == nullptr) {
			return b;
		} else if (b == nullptr) {
			return a;
		} else if (a->p > b->p) {
			a->push_lazy();
			a->cut_r();

			a->set_r(merge(a->r, b));
			a->recalc();
			return a;
		} else {
			b->push_lazy();
			b->cut_l();

			b->set_l(merge(a, b->l));
			b->recalc();
			return b;
		}
	}

	//splits such that the left sub tree has s nodes
	static std::pair<Treap*, Treap*> split(Treap *a, uint32_t s) {
		if (a == nullptr) {
			return {nullptr, nullptr};
		}
		uint32_t lSz = a->l ? a->l->size + 1 : 1;
		if (lSz > s) {
			a->push_lazy();
			a->cut_l();

			auto sp = split(a->l, s);
			a->set_l(sp.second);
			a->recalc();
			return {sp.first, a};
		} else {
			a->push_lazy();
			a->cut_r();

			auto sp = split(a->r, s - lSz);
			a->set_r(sp.first);
			a->recalc();
			return {a, sp.second};
		}
	}
	//the number of nodes before this
	uint32_t get_ind() const {
		uint32_t ans = l ? l->size : 0;

		const Treap *prev = this;
		const Treap *cur = par;

		while (cur != nullptr) {
			if (cur->r == prev) {
				ans++;
				if (cur->l) {
					ans += cur->l->size;
				}
			}
			prev = cur;
			cur = cur->par;
		}
		return ans;
	}
	void print_infix()  {
		push_lazy();
		if (l) {
			assert(l->par == this);
			l->print_infix();
		}
		std::cerr << "(" << cur.i << " " << cur.d <<  ") ";
		if (r) {
			assert(r->par == this);
			r->print_infix();
		}
	}
};
std::mt19937 Treap::rnd(27071999);
Treap Treap::mem[Treap::MAX_MEM];
Treap* Treap::memPtr = Treap::mem;

Treap* treeN[MAX_N];
Treap* treeP[MAX_N];
Treap *root = nullptr;

struct Vertex {
	std::vector<Vertex*> edges;
	Vertex *par;
	uint32_t depth;
	uint32_t ind;

	Vertex(): par(nullptr), depth(0) {}
	
	void root_tree() {
		for (Vertex *e : edges) {
			e->par = this;
			e->depth = depth + 1;
			e->root_tree();
		}
	}

	void build_treap() {
		treeN[ind] = Treap::get_node();
		treeN[ind]->set_cur({ind, (int32_t) depth});
		root = Treap::merge(root, treeN[ind]);

		for (Vertex *e : edges) {
			e->build_treap();

			Treap *n = treeP[e->ind] = Treap::get_node();
			n->set_cur({ind, (int32_t) depth});
			root = Treap::merge(root, n);
		}
	}
};

Vertex tree[MAX_N];
uint32_t vCnt;

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> vCnt;
	for (uint32_t i = 1;i < vCnt;i++) {
		uint32_t a, b;
		std::cin >> a >> b;
		a--;
		b--;

		tree[a].edges.push_back(&tree[b]);
	}
	for (uint32_t i = 0;i < vCnt;i++) {
		tree[i].ind = i + 1;
	}
}

uint32_t query(uint32_t a, uint32_t b) {
	uint32_t aInd = treeN[a]->get_ind();
	uint32_t bInd = treeN[b]->get_ind();

	//std::cerr << aInd << " " << bInd << "\n";

	if (aInd > bInd) {
		std::swap(aInd, bInd);
	}

	auto s1 = Treap::split(root, aInd);
	auto s2 = Treap::split(s1.second, bInd - aInd + 1);
	uint32_t ans = s2.first->get_best().i;

	root = Treap::merge(s1.first, Treap::merge(s2.first, s2.second));
	return ans;
}

void update(uint32_t a, uint32_t b) {
	if (a == b) {
		return;
	}
	uint32_t bStart = treeN[b]->get_ind();
	uint32_t bEnd = treeP[b]->get_ind();

	//std::cerr << "bStart " << bStart << "bend " << bEnd << "\n";

	auto s1 = Treap::split(root, bStart);
	auto s2 = Treap::split(s1.second, bEnd - bStart);
	auto s3 = Treap::split(s2.second, 1);
	
	treeN[a]->prop_lazy();
	int32_t aDepth = treeN[a]->get_cur().d;
	treeP[b]->set_cur({a, aDepth});

	int32_t bDepth = s2.first->get_best().d;
	s2.first->add_lazy(aDepth - bDepth + 1);
	//std::cerr << "b subtrree: ";
	//s2.first->print_infix();
	//std::cerr << "\n";

	Treap *bTreap = Treap::merge(s2.first, s3.first);
	root = Treap::merge(s1.first, s3.second);
	
	uint32_t aInd = treeN[a]->get_ind();
	auto s4 = Treap::split(root, aInd + 1);
	root = Treap::merge(s4.first, Treap::merge(bTreap, s4.second));
	assert(root->get_size() == 2 * vCnt - 1);
}

int main() {
	input();
	tree[0].root_tree();
	tree[0].build_treap();
	//root->print_infix();
//	for (uint32_t i = 1;i <= vCnt;i++) {
//		std::cerr << "Vertex " << i << " Base Ind " << treeN[i]->get_ind();
//		if (i > 1) {
//			std::cerr << " Par Ind " <<  treeP[i]->get_ind();
//		} 
//		std::cerr << "\n";
//	}

	uint32_t qCnt;
	std::cin >> qCnt;
	for (uint32_t i = 0;i < qCnt;i++) {
		uint32_t t, a, b;
		std::cin >> t >> a >> b;
		if (t == 2) {
			std::cout << query(a, b) << "\n";
		} else {
			update(b, a);
			//root->print_infix();
		}
	}
}
