#include <bits/stdc++.h>

const int32_t MAX_N = 1e5;
std::mt19937 mt(69);

struct Vertex {
	int32_t prior, siz, lazy;
	std::pair< int32_t, int32_t > val, bestVal;
	Vertex *l, *r, *par;

	Vertex() {}
	Vertex(std::pair< int32_t, int32_t > _val) : val(_val) {
		siz = 1;
		lazy = 0;
		prior = mt();

		l = nullptr;
		r = nullptr;
		par = nullptr;

		bestVal = val;
	}

	static int32_t get_size(Vertex *x) {
		if(x == nullptr) {
			return 0;
		}

		return x->siz;
	}

	static void update(Vertex *x) {
		x->bestVal = x->val;
		x->siz = 1;
		if(x->l != nullptr) {
			x->l->par = x;
			x->bestVal = std::min(x->bestVal, x->l->bestVal);
			x->siz += x->l->siz;
		}
		if(x->r != nullptr) {
			x->r->par = x;
			x->bestVal = std::min(x->bestVal, x->r->bestVal);
			x->siz += x->r->siz;
		}
	}

	static void push_lazy(Vertex *x) {
		if(x == nullptr) {
			return;
		}

		x->val.first += x->lazy;
		x->bestVal.first += x->lazy;

		if(x->l != nullptr) {
			x->l->lazy += x->lazy;
		}
		if(x->r != nullptr) {
			x->r->lazy += x->lazy;
		}

		x->lazy = 0;
	}
};

bool first[MAX_N + 5];
int32_t n;
std::vector< int32_t > g[MAX_N + 5];
std::vector< std::pair< int32_t, int32_t > > dfsOrder;
Vertex *root = nullptr, *leftVal[MAX_N + 5], *rightVal[MAX_N + 5];

std::pair< Vertex*, Vertex* > split(Vertex *v, int32_t key, int32_t add) {
	if(v == nullptr) {
		return { nullptr, nullptr };
	}

	Vertex::push_lazy(v);

	int32_t currKey = Vertex::get_size(v->l) + add;
	if(currKey < key) {
		auto aux = split(v->r, key, add + Vertex::get_size(v->l) + 1);
		v->r = aux.first;
		Vertex::update(v);
		return { v, aux.second };
	}
	else {
		auto aux = split(v->l, key, add);
		v->l = aux.second;
		Vertex::update(v);
		return { aux.first, v };
	}
}

Vertex* merge(Vertex *a, Vertex *b) {
	Vertex::push_lazy(a);
	Vertex::push_lazy(b);

	if(a == nullptr) {
		return b;
	}
	if(b == nullptr) {
		return a;
	}

	if(a->prior > b->prior) {
		a->r = merge(a->r, b);
		Vertex::update(a);
		return a;
	}
	else {
		b->l = merge(a, b->l);
		Vertex::update(b);
		return b;
	}
}

Vertex* insert(Vertex *v, Vertex *newV, int32_t ind) {
	if(v == nullptr) {
		v = newV;
		return v;
	}
	
	auto aux = split(v, ind, 0);
	aux.second = merge(newV, aux.second);
	v = merge(aux.first, aux.second);
	return v;
}

std::pair< Vertex*, Vertex* > remove(Vertex *v, int32_t low, int32_t high) {
	auto aux1 = split(v, low, 0);
	auto aux2 = split(aux1.second, high - low + 1, 0);
	aux1.second = aux2.second;
	v = merge(aux1.first, aux1.second);
	return { v, aux2.first };
}

void compute_dfs_order(int32_t v, int32_t par, int32_t dep) {
	dfsOrder.push_back({ dep, v });

	for(auto &x : g[v]) {
		if(x == par) {
			continue;
		}

		compute_dfs_order(x, v, dep + 1);
		dfsOrder.push_back({ dep, v });
	}
	
	if(g[v].size() == 1) {
		dfsOrder.push_back({ dep, v });
	}
}

void print_treap(Vertex *v) {
	if(v == nullptr) {
		return;
	}
	
	print_treap(v->l);
	std::cout << "val = { " << v->val.first << ", " << v->val.second << " }, size = " << Vertex::get_size(v) << '\n';
	print_treap(v->r);
}

int32_t get_ind(Vertex *v, bool p = true) {
	if(v->par == nullptr) {
		if(p) {
			return Vertex::get_size(v->l);
		}
		else {
			return 0;
		}
	}

	if(v->par->l == v) {
		return get_ind(v->par, false) + (p ? Vertex::get_size(v->l) : 0);
	}
	else {
		return get_ind(v->par, true) + 1 + (p ? Vertex::get_size(v->l) : 0);
	}
}

void build() {
	compute_dfs_order(1, -1, 0);
	for(auto &x : dfsOrder) {
		Vertex *v = new Vertex(x);
		root = insert(root, v, Vertex::get_size(root));
		
		if(!first[x.second]) {
			leftVal[x.second] = v;
			first[x.second] = true;
		}
		else {
			rightVal[x.second] = v;
		}
	}
}

std::pair< int32_t, int32_t > get_val(Vertex *v, int32_t ind) {
	Vertex::push_lazy(v);
	if(Vertex::get_size(v->l) == ind) {
		return v->val;
	}

	if(Vertex::get_size(v->l) > ind) {
		return get_val(v->l, ind);
	}
	else {
		return get_val(v->r, ind - Vertex::get_size(v->l) - 1);
	}
}

void update(int32_t low, int32_t high, int32_t ind) {
	assert(low != 0);
	std::pair< int32_t, int32_t > prevVal = get_val(root, low - 1);
	std::pair< int32_t, int32_t > newVal = get_val(root, ind);
	auto aux = remove(root, low, high);
		
	if(ind > high) {
		ind -= (high - low + 1);
	}

	root = aux.first;
	aux.second->lazy += (newVal.first - prevVal.first);
	Vertex::push_lazy(aux.second);
	aux.second = merge(aux.second, new Vertex(newVal));
	root = insert(root, aux.second, ind + 1);
}

int32_t query(int32_t low, int32_t high) {
	auto aux1 = split(root, low, 0);
	auto aux2 = split(aux1.second, high - low + 1, 0);
	assert(aux2.first->lazy == 0);
	Vertex::push_lazy(aux2.first);
	int32_t ans = aux2.first->bestVal.second;
	aux1.second = merge(aux2.first, aux2.second);
	root = merge(aux1.first, aux1.second);
	return ans;
}

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> n;

	for(int32_t i = 0; i < n - 1; i++) {
		int32_t x, y;
		std::cin >> x >> y;

		g[x].push_back(y);
		g[y].push_back(x);
	}

	build();

	int32_t q;
	std::cin >> q;

	for(int32_t i = 0; i < q; i++) {
		int32_t type, x, y;
		std::cin >> type >> x >> y;
		
		int32_t leftX = get_ind(leftVal[x]);
		int32_t rightX = get_ind(rightVal[x]);
		int32_t leftY = get_ind(leftVal[y]);
		int32_t rightY = get_ind(rightVal[y]);
		
		assert(get_val(root, leftX).second == x && get_val(root, leftY).second == y && get_val(root, rightX).second == x && get_val(root, rightY).second == y);

		if(type == 1) {
			assert(leftY < leftX || leftY > rightX);
			update(leftX, rightX, leftY);	
		}
		else {
			if(leftX <= leftY && rightX >= rightY) {
				std::cout << x << '\n';
			}
			else if(leftX >= leftY && rightX <= rightY) {
				std::cout << y << '\n';
			}
			else {
				int32_t ans = query(std::min(rightX, rightY), std::max(leftX, leftY));
				assert(ans >= 1 && ans <= n && ans != x && ans != y);
				std::cout << ans << '\n';
			}
		}
	}
}
