#include <iostream>
#include <random>
#include <vector>
#include <algorithm>
#include <utility>
#define fi first
#define se second
using namespace std;

const int MAXN = 100005;

mt19937 rg(4209008);

int n, q;
vector<int> g[MAXN];
int level[MAXN];

struct node
{
    int val, prior, cnt;
    int lev, lmin, vmin;
    int lazy;

    node *l, *r, *p;

    node(int val, int lev)
    {
        this -> val = val;
        this -> lev = lev;
        this -> cnt = 1;
        this -> prior = rg();
        this -> lmin = lev;
        this -> vmin = val;
        this -> lazy = 0;

        this -> l = this -> r = nullptr;
        this -> p = nullptr;
    }

    void oper()
    {
        this -> cnt = 1;
        this -> lev += lazy;
        this -> lmin = lev;
        this -> vmin = val;

        if (this -> l != nullptr)
        {
            cnt += l -> cnt;
            l -> p = this;
            if (lazy != 0) l -> lazy += lazy;
            if (l -> lmin + l -> lazy < this -> lmin)
            {
                this -> lmin = l -> lmin + l -> lazy;
                this -> vmin = l -> vmin;
            }
        }
        if (this -> r != nullptr)
        {
            cnt += r -> cnt;
            r -> p = this;
            if (lazy != 0) r -> lazy += lazy;
            if (r -> lmin + r -> lazy < this -> lmin)
            {
                this -> lmin = r -> lmin + r -> lazy;
                this -> vmin = r -> vmin;
            }
        }

        this -> lazy = 0;
    }
};

node * root = nullptr;
node * m[MAXN][2];

pair<node *, node *> Split(node * t, int pos)
{
    if (t == nullptr) return {nullptr, nullptr};
    else t -> oper();

    int lsize = (t -> l != nullptr) ? t -> l -> cnt : 0;

    if (pos <= lsize)
    {
        auto res = Split(t -> l, pos);
        t -> l = res.se;
        t -> oper();
        return {res.fi, t};
    }
    else
    {
        auto res = Split(t -> r, pos - lsize - 1);
        t -> r = res.fi;
        t -> oper();
        return {t, res.se};
    }
}

node * Merge(node * sm, node * bg)
{
    if (sm == nullptr) return bg;
    if (bg == nullptr) return sm;

    if (sm -> prior > bg -> prior)
    {
        sm -> r = Merge(sm -> r, bg);
        sm -> oper();
        return sm;
    }
    else
    {
        bg -> l = Merge(sm, bg -> l);
        bg -> oper();
        return bg;
    }
}

void add(int val, int lev)
{
    if (root == nullptr)
    {
        root = new node(val, lev);
        if (m[val][0] == nullptr) m[val][0] = root;
        m[val][1] = root;
    }
    else
    {
        node * nv = new node(val, lev);
        if (m[val][0] == nullptr) m[val][0] = nv;
        m[val][1] = nv;

        root = Merge(root, nv);
    }
}

int get_pos(node * t)
{
    if (t == nullptr)
    {
        cout << "need to find pos of nullptr" << endl;
        return 0;
    }

    t -> oper();
    int pos = 1;
    if (t -> l != nullptr) pos += t -> l -> cnt;

    node * pre = t;
    t = t -> p;
    while (t != nullptr)
    {
        if (t -> r != nullptr and t -> r == pre)
        {
            pos++;
            if (t -> l != nullptr) pos += t -> l -> cnt;
        }
        pre = t;
        t = t -> p;
    }

    return pos;
}

void reorder(int l, int r, int pos, int delta, int vertex)
{
  ///  cout << "reordering " << l << ' ' <<r << endl;

    if (l <= pos and pos <= r)
    {
        cout << "cannot move the part in it" << endl;
        return;
    }

    auto tmp1 = Split(root, r);
    auto tmp2 = Split(tmp1.fi, l - 1); // then parts are tmp2.fi, tmp2.se, tmp1.se

    node * nv = new node(vertex, level[vertex]);
    tmp2.se -> lazy += delta;
    tmp2.se = Merge(tmp2.se, nv);


    if (pos < l)
    {
        auto tmp3 = Split(tmp2.fi, pos);
        tmp3.fi = Merge(tmp3.fi, tmp2.se);
        tmp2.fi = Merge(tmp3.fi, tmp3.se);
        root = Merge(tmp2.fi, tmp1.se);
    }
    else
    {
        pos -= r;
        auto tmp3 = Split(tmp1.se, pos);
        tmp3.fi = Merge(tmp3.fi, tmp2.se);
        tmp1.se = Merge(tmp3.fi, tmp3.se);
        root = Merge(tmp2.fi, tmp1.se);
    }

    // to check
}

int get_level(int u)
{
    int x = get_pos(m[u][0]);

    auto tmp1 = Split(root, x);
    auto tmp2 = Split(tmp1.fi, x - 1);

    int lev = tmp2.se -> lev;

    tmp1.fi = Merge(tmp2.fi, tmp2.se);
    root = Merge(tmp1.fi, tmp1.se);

    return lev;
}



void Init()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    cin >> n;
    for (int i = 1; i < n; ++ i)
    {
        int u, v;
        cin >> u >> v;
        g[u].push_back(v);
    }

    for (int i = 0; i < MAXN; ++ i)
    {
        m[i][0] = nullptr;
        m[i][1] = nullptr;
    }
}

void preorder(node * t)
{

    if (t == nullptr) return;
    t-> oper();

   // cout << "left" << endl;
    preorder(t -> l);
   // cout << "right" << endl;
 ///  cout << t -> val << ' ';
    preorder(t -> r);
    //cout << "end" << endl;
}

void update(int u, int v)
{
    int delta = get_level(u) + 1 - get_level(v);

    int l = get_pos(m[v][0]);




    int r = get_pos(m[v][1]);
    int pos = get_pos(m[u][0]);

 ///   cout << "query for " << u << ' ' << v << endl;

 ///   cout << l << ' ' << r << endl;

  ///  cout << pos << endl;


    reorder(l, r, pos, delta, u);
}

int query(int u, int v)
{
    int l = min(get_pos(m[u][0]), get_pos(m[v][0]));
    int r = max(get_pos(m[u][1]), get_pos(m[v][1]));

 ///   cout << l << ' ' << r << endl;


    auto tmp1 = Split(root, r);
    auto tmp2 = Split(tmp1.fi, l - 1);

    int ans = tmp2.se -> vmin;
    //cout << tmp2.se -> lmin << endl;

    tmp1.fi = Merge(tmp2.fi, tmp2.se);
    root = Merge(tmp1.fi, tmp1.se);

    return ans;
}

void DFS(int u = 1, int l = 1)
{
   /// cout << "(" << u << ", " << l << ")" << ' ';
    add(u, l);
    for (int v : g[u])
    {
        DFS(v, l + 1);
        //cout << u << ' ';
     ///   cout << "(" << u << ", " << l << ")" << ' ';
        add(u, l);
    }
}

int main()
{
    Init();
    DFS();

  ///  cout << endl;

  ///  preorder(root) ;

///cout << endl;

    cin >> q;
    for (int i = 1; i <= q; ++ i)
    {
        int t;
        cin >> t;

        if (t == 1)
        {
            int x, y;
            cin >> x >> y;
            update(y, x);
        }
        if (t == 2)
        {
            int x, y;
            cin >> y >> x;
            cout  << query(x, y) << endl;
        }

     ///   preorder(root);
       /// cout << endl;
    }

    return 0;
}

/*
5
1 2
1 3
2 4
2 5
8
2 4 3
2 5 5
2 1 2
1 2 3
2 4 3
2 4 5
1 4 3
2 4 5
*/
